from .exceptions import *
from .apis import *
from .apis.fyle_v3 import FyleV3
import requests
import json


class FyleSDK:
    """The main class which creates a connection with Fyle APIs using OAuth2 authentication (refresh token grant type).

    Parameters:
        client_id (str): Client ID for Fyle API.
        client_secret (str): Client secret for Fyle API.
        refresh_token (str): Refresh token for Fyle API.
    """

    TOKEN_URL = '{}/api/oauth/token'

    def __init__(self, base_url, client_id, client_secret, refresh_token, origin_address=None):
        # store the credentials
        self.__base_url = base_url
        self.__client_id = client_id
        self.__client_secret = client_secret
        self.__refresh_token = refresh_token
        self.__origin_address = origin_address
        self.access_token = None

        # # V3 Instance
        self.v3 = FyleV3()

        # V1 APIs
        self.Employees = Employees()
        self.Expenses = Expenses()
        self.Reports = Reports()
        self.Categories = Categories()
        self.Advances = Advances()
        self.AdvanceRequests = AdvanceRequests()
        self.Refunds = Refunds()
        self.Reimbursements = Reimbursements()
        self.Settlements = Settlements()
        self.CostCenters = CostCenters()
        self.Projects = Projects()
        self.BalanceTransfers = BalanceTransfers()
        self.Exports = Exports()
        self.TripRequests = TripRequests()
        self.TransportationRequests = TransportationRequests()
        self.TransportationBookings = TransportationBookings()
        self.TransportationBookingCancellations = TransportationBookingCancellations()
        self.HotelRequests = HotelRequests()
        self.HotelBookings = HotelBookings()
        self.HotelBookingCancellations = HotelBookingCancellations()
        self.CorporateCreditCardExpenses = CorporateCreditCardExpenses()
        self.Files = Files()
        self.Orgs = Orgs()
        self.ExpensesCustomFields = ExpensesCustomFields()

        # get the access token
        self.update_access_token()
        self.set_server_url()
        self.set_origin_address()

    def set_origin_address(self):
        """Set the Client Origin IP Address in all API objects."""

        origin_address = self.__origin_address

        # V3 APIs
        self.v3.expenses.set_origin_address(origin_address)
        self.v3.reports.set_origin_address(origin_address)
        self.v3.employees.set_origin_address(origin_address)
        self.v3.orgs.set_origin_address(origin_address)
        self.v3.reimbursements.set_origin_address(origin_address)
        self.v3.cost_centers.set_origin_address(origin_address)
        self.v3.categories.set_origin_address(origin_address)
        self.v3.projects.set_origin_address(origin_address)
        self.v3.refunds.set_origin_address(origin_address)
        self.v3.balance_transfers.set_origin_address(origin_address)
        self.v3.settlements.set_origin_address(origin_address)
        self.v3.advances.set_origin_address(origin_address)
        self.v3.advance_requests.set_origin_address(origin_address)
        self.v3.bank_transactions.set_origin_address(origin_address)
        self.v3.trip_requests.set_origin_address(origin_address)
        self.v3.expense_custom_properties.set_origin_address(origin_address)
        self.v3.employee_custom_properties.set_origin_address(origin_address)
        self.v3.trip_request_custom_properties.set_origin_address(origin_address)
        self.v3.advance_request_custom_properties.set_origin_address(origin_address)

        # V1 APIs
        self.Employees.set_origin_address(origin_address)
        self.Expenses.set_origin_address(origin_address)
        self.Reports.set_origin_address(origin_address)
        self.Categories.set_origin_address(origin_address)
        self.Advances.set_origin_address(origin_address)
        self.AdvanceRequests.set_origin_address(origin_address)
        self.Refunds.set_origin_address(origin_address)
        self.Reimbursements.set_origin_address(origin_address)
        self.Settlements.set_origin_address(origin_address)
        self.CostCenters.set_origin_address(origin_address)
        self.Projects.set_origin_address(origin_address)
        self.BalanceTransfers.set_origin_address(origin_address)
        self.Exports.set_origin_address(origin_address)
        self.TripRequests.set_origin_address(origin_address)
        self.TransportationRequests.set_origin_address(origin_address)
        self.TransportationBookings.set_origin_address(origin_address)
        self.TransportationBookingCancellations.set_origin_address(origin_address)
        self.HotelRequests.set_origin_address(origin_address)
        self.HotelBookings.set_origin_address(origin_address)
        self.HotelBookingCancellations.set_origin_address(origin_address)
        self.CorporateCreditCardExpenses.set_origin_address(origin_address)
        self.Files.set_origin_address(origin_address)
        self.Orgs.set_origin_address(origin_address)
        self.ExpensesCustomFields.set_origin_address(origin_address)

    def update_access_token(self):
        """Update the access token and change it in all API objects."""

        access_token = self.__get_access_token()

        # V3 APIs
        self.v3.expenses.change_access_token(access_token)
        self.v3.reports.change_access_token(access_token)
        self.v3.employees.change_access_token(access_token)
        self.v3.orgs.change_access_token(access_token)
        self.v3.reimbursements.change_access_token(access_token)
        self.v3.cost_centers.change_access_token(access_token)
        self.v3.categories.change_access_token(access_token)
        self.v3.projects.change_access_token(access_token)
        self.v3.refunds.change_access_token(access_token)
        self.v3.balance_transfers.change_access_token(access_token)
        self.v3.settlements.change_access_token(access_token)
        self.v3.advance_requests.change_access_token(access_token)
        self.v3.advances.change_access_token(access_token)
        self.v3.bank_transactions.change_access_token(access_token)
        self.v3.trip_requests.change_access_token(access_token)
        self.v3.expense_custom_properties.change_access_token(access_token)
        self.v3.employee_custom_properties.change_access_token(access_token)
        self.v3.trip_request_custom_properties.change_access_token(access_token)
        self.v3.advance_request_custom_properties.change_access_token(access_token)

        # V1 APIs
        self.Employees.change_access_token(access_token)
        self.Expenses.change_access_token(access_token)
        self.Reports.change_access_token(access_token)
        self.Categories.change_access_token(access_token)
        self.Advances.change_access_token(access_token)
        self.AdvanceRequests.change_access_token(access_token)
        self.Refunds.change_access_token(access_token)
        self.Reimbursements.change_access_token(access_token)
        self.Settlements.change_access_token(access_token)
        self.CostCenters.change_access_token(access_token)
        self.Projects.change_access_token(access_token)
        self.BalanceTransfers.change_access_token(access_token)
        self.Exports.change_access_token(access_token)
        self.TripRequests.change_access_token(access_token)
        self.TransportationRequests.change_access_token(access_token)
        self.TransportationBookings.change_access_token(access_token)
        self.TransportationBookingCancellations.change_access_token(access_token)
        self.HotelRequests.change_access_token(access_token)
        self.HotelBookings.change_access_token(access_token)
        self.HotelBookingCancellations.change_access_token(access_token)
        self.CorporateCreditCardExpenses.change_access_token(access_token)
        self.Files.change_access_token(access_token)
        self.Orgs.change_access_token(access_token)
        self.ExpensesCustomFields.change_access_token(access_token)

    def set_server_url(self):
        """Set the Base URL in all API objects."""

        base_url = self.__base_url

        # V3 APIs
        self.v3.expenses.set_server_url(base_url)
        self.v3.reports.set_server_url(base_url)
        self.v3.employees.set_server_url(base_url)
        self.v3.orgs.set_server_url(base_url)
        self.v3.reimbursements.set_server_url(base_url)
        self.v3.cost_centers.set_server_url(base_url)
        self.v3.categories.set_server_url(base_url)
        self.v3.projects.set_server_url(base_url)
        self.v3.refunds.set_server_url(base_url)
        self.v3.balance_transfers.set_server_url(base_url)
        self.v3.settlements.set_server_url(base_url)
        self.v3.advances.set_server_url(base_url)
        self.v3.advance_requests.set_server_url(base_url)
        self.v3.bank_transactions.set_server_url(base_url)
        self.v3.trip_requests.set_server_url(base_url)
        self.v3.expense_custom_properties.set_server_url(base_url)
        self.v3.employee_custom_properties.set_server_url(base_url)
        self.v3.trip_request_custom_properties.set_server_url(base_url)
        self.v3.advance_request_custom_properties.set_server_url(base_url)

        # V1 APIs
        self.Employees.set_server_url(base_url)
        self.Expenses.set_server_url(base_url)
        self.Reports.set_server_url(base_url)
        self.Categories.set_server_url(base_url)
        self.Advances.set_server_url(base_url)
        self.AdvanceRequests.set_server_url(base_url)
        self.Refunds.set_server_url(base_url)
        self.Reimbursements.set_server_url(base_url)
        self.Settlements.set_server_url(base_url)
        self.CostCenters.set_server_url(base_url)
        self.Projects.set_server_url(base_url)
        self.BalanceTransfers.set_server_url(base_url)
        self.Exports.set_server_url(base_url)
        self.TripRequests.set_server_url(base_url)
        self.TransportationRequests.set_server_url(base_url)
        self.TransportationBookings.set_server_url(base_url)
        self.TransportationBookingCancellations.set_server_url(base_url)
        self.HotelRequests.set_server_url(base_url)
        self.HotelBookings.set_server_url(base_url)
        self.HotelBookingCancellations.set_server_url(base_url)
        self.CorporateCreditCardExpenses.set_server_url(base_url)
        self.Files.set_server_url(base_url)
        self.Orgs.set_server_url(base_url)
        self.ExpensesCustomFields.set_server_url(base_url)

    def __get_access_token(self):
        """Get the access token using a HTTP post.
        
        Returns:
            A new access token.
        """

        api_data = {
            'grant_type': 'refresh_token',
            'refresh_token': self.__refresh_token,
            'client_id': self.__client_id,
            'client_secret': self.__client_secret
        }

        token_url = FyleSDK.TOKEN_URL.format(self.__base_url)
        response = requests.post(token_url, data=api_data)

        if response.status_code == 200:
            access_token = json.loads(response.text)['access_token']
            self.access_token = access_token
            return access_token

        elif response.status_code == 401:
            raise UnauthorizedClientError('Wrong client secret or/and refresh token', response.text)

        elif response.status_code == 404:
            raise NotFoundClientError('Client ID doesn\'t exist', response.text)

        elif response.status_code == 500:
            raise InternalServerError('Internal server error', response.text)

        else:
            raise FyleSDKError('Error: {0}'.format(response.status_code), response.text)